"""
main
"""
import os
from typing import Dict, Any, List
from pytconf import register_endpoint, register_main, config_arg_parse_and_launch

from termcolor import cprint
from pypowerline.utils import execute_python_file
from pypowerline.segments import Segment

from pypowerline.symbols import Symbol
from pypowerline.static import DESCRIPTION, APP_NAME, VERSION_STR


@register_endpoint(
    configs=[],
    description="print a prompt for bash",
)
def bash() -> None:
    py_file = os.path.expanduser("~/.config/pypowerline/segments.py")
    vals: Dict[str, Any] = {}
    try:
        execute_python_file(py_file, vals=vals)
    # pylint: disable=broad-exception-caught
    except Exception as e:
        print(e)
        print("cannot execute segments > ", end="")
        return
    if "segments" not in vals:
        print("segments not defined > ", end="")
        return
    segments: List[Segment] = vals["segments"]
    for segment in segments:
        if segment.show_icon:
            print(segment.get_icon().value, " ", end="")
        print(segment.get_text(), end="")


@register_endpoint(
    configs=[],
    description="do some test printing on the terminal (development purposes)",
)
def test() -> None:
    cwd = os.getcwd()
    symbol = Symbol.SEPARATOR_B.value
    print(f"{cwd} > ", end="")
    cprint("cwd", "black", "on_light_magenta", end="")
    cprint("cwd", "black", "on_light_grey", end="")
    cprint("cwd", "black", "on_grey", end="")
    cprint("cwd", "black", "on_blue", end="")
    cprint("cwd", "black", "on_light_blue", end="")
    cprint("cwd", "white", "on_cyan", end="")
    cprint("cwd", "white", "on_green", end="")
    cprint(f"{symbol}", "light_cyan", "on_black", end="")
    print()


@register_endpoint(
    configs=[],
    description="do tmux",
)
def tmux() -> None:
    cwd = os.getcwd()
    print(f"{cwd} > ", end="")
    cprint(f"{cwd}", "red", "on_light_magenta", end="")


@register_endpoint(
    configs=[],
    description="print special symbols (used for development)",
)
def dump_symbols() -> None:
    for symbol in Symbol:
        print(symbol, symbol.value)


@register_main(
    main_description=DESCRIPTION,
    app_name=APP_NAME,
    version=VERSION_STR,
)
def main():
    config_arg_parse_and_launch()


if __name__ == "__main__":
    main()



from abc import ABC, abstractmethod

import os
import os.path

from pypowerline.symbols import Symbol


class Segment(ABC):
    def __init__(self, show_icon: bool = True):
        self.show_icon = show_icon

    @abstractmethod
    def get_text(self) -> str:
        """ return the text of the segment """

    def get_icon(self) -> Symbol:
        """ return the special character of the segment """
        return Symbol.NONE

    # def get_color(self) -> str:
    #     """ return the color of the segment """
    # def get_background_color(self) -> str:
    #     """ return the background color of the segment """


class SegmentCwd(Segment):
    def __init__(
            self,
            color: str = "green",
            last_only: bool = False,
            home_as_tilde: bool = True,
            show_icon: bool = True,
    ):
        super().__init__(show_icon=show_icon)
        self.color = color
        self.last_only = last_only
        self.home_as_tilde = home_as_tilde
        self.home_directory = os.path.expanduser("~")

    def get_text(self) -> str:
        cwd = os.getcwd()
        if self.home_as_tilde and cwd.startswith(self.home_directory):
            cwd = "~" + cwd[len(self.home_directory):]
        if self.last_only:
            return os.path.basename(cwd)
        return cwd

    def get_icon(self) -> Symbol:
        return Symbol.FOLDER


class SegmentForward(Segment):
    def __init__(self):
        super().__init__(show_icon=False)

    def get_text(self):
        return " > "



from enum import Enum


class Symbol(Enum):
    NONE = "None"
    DETACHED = "\u2693"
    AHEAD = "\u2B06"
    BEHIND = "\u2B07"
    STAGED = "\u2714"
    CHANGED = "\u270E"
    NEW = "?"
    CONFLICTED = "\u273C"
    STASH = "\u2398"
    GIT = "\uE0A0"
    HG = "\u263F"
    BZR = "\u2B61\u20DF"
    FOSSIL = "\u2332"
    SVN = "\u2446"
    SEPARATOR = "\u25B6"
    SEPARATOR_B = "\uE0B0"
    SEPARATOR_THIN = "\u276F"
    SEPARATOR_THIN_B = "\uE0B1"
    LOCK = "\uE0A2"
    FOLDER = "\U0001F5C1"
    VIRTUAL_ENV = "\U0001F40D"
    BATTERY = "\U0001F50C"
    ELLIPSIS = "\u2026"
    BAT = "\u26A1"
    UP = "\u2191"



"""
All configurations for pypowerline
"""
from pytconf import Config, ParamCreator


class ConfigGeneral(Config):
    """
    Parameters for configuring the separator
    """
    icons = ParamCreator.create_bool(
        help_string="Should we show icons?",
        default=True,
    )
    colors = ParamCreator.create_bool(
        help_string="Should we use colors?",
        default=True,
    )



def execute_python_file(file_path: str, vals=None):
    try:
        with open(file_path, 'r') as file:
            python_code = file.read()
            # pylint: disable=exec-used
            exec(python_code, globals(), vals)
    except FileNotFoundError:
        print(f"Error: The file [{file_path}] does not exist.")
